﻿

using BPA.KitChen.GroupMealOrder.Application.BaseDto;
using BPA.KitChen.GroupMealOrder.Application.Service.WeighOrder.Dtos;
using BPA.KitChen.GroupMealOrder.Application.Service.WeighOrder.Service;
using BPA.KitChen.GroupMealOrder.Core;
using Furion.DynamicApiController;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace BPA.KitChen.GroupMealOrder.Application.Service.AExternalPlatform.Service.WeighOrder
{

    [ApiDescriptionSettings("开放平台", Tag = "称重订单"), AllowAnonymous]
    public class WeighOrderServices: IDynamicApiController
    {
        private readonly IWeighOrderService _weighOrderService;
        public WeighOrderServices(IWeighOrderService weighOrderService)
        {
            _weighOrderService = weighOrderService;
        }

        /// <summary>
        /// 创建订单
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/ExternalPlatform/WeighOrder/CreateWeighOrder")]
        public async Task<WeighOrderDto> CreateWeighOrder(WeighOrderCreteDto inputDto)
        {
            return await _weighOrderService.CreateWeighOrder(inputDto);
        }

        /// <summary>
        /// 添加订单商品
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/ExternalPlatform/WeighOrder/AddWeighOrderGoods")]
        public async Task<bool> AddWeighOrderGoods(List<WeighOrderGoodsCreateDto> inputDto)
        {
            return await _weighOrderService.AddWeighOrderGoods(inputDto);
        }

        /// <summary>
        /// 添加订单餐盘
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost( "/api/ExternalPlatform/WeighOrder/AddWeighOrderDiningPlate")]
        public async Task<bool> AddWeighOrderDiningPlate(List<WeighOrderDiningPlateCreateDto> inputDto)
        {
            return await _weighOrderService.AddWeighOrderDiningPlate(inputDto);
        }

        /// <summary>
        /// 查询订单商品
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost( "/api/ExternalPlatform/WeighOrder/GetWeighOrder")]
        public async Task<WeighOrderDto> GetWeighOrder(GetOrderInputDto inputDto)
        {
            return await _weighOrderService.GetWeighOrder(inputDto);
        }

        /// <summary>
        /// 修改订单状态
        /// </summary>
        /// <returns></returns>
        [HttpPost( "/api/ExternalPlatform/WeighOrder/UpdateWeighOrderStates")]
        public async Task<bool> UpdateWeighOrderStates(WeighOrderUpdateDto inputDto)
        {

            return await _weighOrderService.UpdateWeighOrderStates(inputDto);
        }

        /// <summary>
        /// 更具openid获取称重订单
        /// </summary>
        /// <returns></returns>
        [HttpPost("/api/ExternalPlatform/WeighOrder/GetWeighOrderByOpenId")]
        public async Task<PageUtil> GetWeighOrderByOpenId(GetWeighOrderByOpenIdInputDto inputDto)
        {
            return await _weighOrderService.GetWeighOrderByOpenId(inputDto);
        }
    }
}
