﻿using BPA.KitChen.GroupMealOrder.Application.BaseDto;
using BPA.KitChen.GroupMealOrder.Application.Service.ThirdAuthorize;
using BPA.KitChen.GroupMealOrder.Application.Service.WeighOrder.Dtos;
using BPA.KitChen.GroupMealOrder.Core;
using BPA.KitChen.GroupMealOrder.Core.Entity;
using BPA.KitChen.GroupMealOrder.SqlSugar;
using Furion.DependencyInjection;
using Furion.DynamicApiController;
using Microsoft.AspNetCore.Components.Forms;
using Microsoft.AspNetCore.Hosting;
using Newtonsoft.Json;
using NPOI.SS.Formula.Functions;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.KitChen.GroupMealOrder.Application.Service.WeighOrder.Service
{
    public class WeighOrderService : ITransient, IWeighOrderService
    {
        private readonly SqlSugarScope db;

        public WeighOrderService()
        {
            db = SqlSugarDb.Db;
        }

        /// <summary>
        /// 创建订单
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<WeighOrderDto> CreateWeighOrder(WeighOrderCreteDto inputDto)
        {
            var result = new WeighOrderDto();



            var order = new BPA_WeighOrder()
            {
                CreateAt = DateTime.Now,
                GroupId = CurrentUser.TenantId,
                CreateId = inputDto.CreateId,
                OrderNumber = Guid.NewGuid().ToString(),
                Id = Guid.NewGuid().ToString(),
                PayStates = 0,
                SceneId = inputDto.SceneId,
                States = 0,
                TotalAmount = inputDto.TotalAmount,
                Subject = inputDto.Subject,
            };
            //添加订单
            var res = await db.Insertable(order).ExecuteCommandAsync();

            var orderGoods = new List<BPA_WeighOrderGoods>();
            foreach (var item in inputDto.Goods)
            {
                orderGoods.Add(new BPA_WeighOrderGoods()
                {
                    GoodsAttribute = "",
                    GoodsId = item.GoodsId,
                    GoodsName = item.GoodsName,
                    GoodsWeight = item.GoodsWeight,
                    GroupId = CurrentUser.TenantId,
                    Id = Guid.NewGuid().ToString(),
                    OrderId = order.Id

                });
            }


            var diningPlate = new List<BPA_WeighOrderDiningPlate>();
            foreach (var item in inputDto.DiningPlateInfo)
            {
                diningPlate.Add(new BPA_WeighOrderDiningPlate()
                {
                    DiningPlateId = item.DiningPlateId,
                    GroupId = CurrentUser.TenantId,
                    WeighOrderId = order.Id,
                    ChipCode = item.ChipCode,
                    QRCode = item.QRCode,
                    Id = Guid.NewGuid().ToString(),
                });
            }

            //添加子订单
            if (res > 0)
            {
                result = new WeighOrderDto()
                {
                    CreateAt = order.CreateAt,
                    CreateId = order.CreateId,
                    GroupId = order.GroupId,
                    Id = order.Id,
                    OrderNumber = order.OrderNumber,
                    PayStates = order.PayStates,
                    SceneId = order.SceneId,
                    States = order.States,
                    Subject = inputDto.Subject,
                    TotalAmount = order.TotalAmount,
                };
                await db.Insertable(orderGoods).ExecuteCommandAsync();
                await db.Insertable(diningPlate).ExecuteCommandAsync();
            }

            result = await GetWeighOrder(new GetOrderInputDto()
            {
                OrderId = order.Id,
            });

            return result;
        }

        /// <summary>
        /// 添加订单商品
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> AddWeighOrderGoods(List<WeighOrderGoodsCreateDto> inputDto)
        {
            var orderGoods = new List<BPA_WeighOrderGoods>();
            foreach (var item in inputDto)
            {
                orderGoods.Add(new BPA_WeighOrderGoods()
                {
                    GoodsAttribute = "",
                    GoodsId = item.GoodsId,
                    GoodsName = item.GoodsName,
                    GoodsWeight = item.GoodsWeight,
                    GroupId = CurrentUser.TenantId,
                    OrderId = item.OrderId,
                    Id = Guid.NewGuid().ToString(),

                });
            }

            var res = await db.Insertable(orderGoods).ExecuteCommandAsync();

            return res > 0;
        }

        /// <summary>
        /// 添加订单餐盘
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> AddWeighOrderDiningPlate(List<WeighOrderDiningPlateCreateDto> inputDto)
        {
            var diningPlate = new List<BPA_WeighOrderDiningPlate>();
            foreach (var item in inputDto)
            {
                diningPlate.Add(new BPA_WeighOrderDiningPlate()
                {
                    DiningPlateId = item.DiningPlateId,
                    GroupId = CurrentUser.TenantId,
                    WeighOrderId = item.OrderId,
                    Id = Guid.NewGuid().ToString(),
                });
            }

            var res = await db.Insertable(diningPlate).ExecuteCommandAsync();
            return res > 0;
        }

        /// <summary>
        /// 查询称重订单信息
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<WeighOrderDto> GetWeighOrder(GetOrderInputDto inputDto)
        {
            var result = new WeighOrderDto();

            var orderIds = new List<string>();

            if (!string.IsNullOrEmpty(inputDto.ChipCode))
            {
                //查询订单餐盘
                 orderIds = await db.Queryable<BPA_WeighOrderDiningPlate>()
                    .Where(x => x.ChipCode == inputDto.ChipCode)
                    .Select(x =>x.WeighOrderId)
                    .ToListAsync();
            ;
            }

            if (!string.IsNullOrEmpty(inputDto.OrderId))
            {
                orderIds.Add(inputDto.OrderId);
            }


            //查询订单
            result = await db.Queryable<BPA_WeighOrder>()
                .WhereIF(!string.IsNullOrEmpty(inputDto.OrderId), x => orderIds.Contains(inputDto.OrderId))
                .Select(x => new WeighOrderDto
                {
                    CreateAt = x.CreateAt,
                    CreateId = x.CreateId,
                    GroupId = x.GroupId,
                    Id = x.Id,
                    OrderNumber = x.OrderNumber,
                    PayStates = 0,
                    SceneId = x.SceneId,
                    States = 0,
                    Subject = x.Subject,
                    TotalAmount = x.TotalAmount,
                })
                .FirstAsync();

            //查询订单商品
            var orderGoods = await db.Queryable<BPA_WeighOrderGoods>()
                .Where(x => x.OrderId == inputDto.OrderId)
                .ToListAsync();

            result.GoodsInfo = orderGoods.Select(x => new WeighOrderGoodsDto()
            {
                GoodsId = x.GoodsId,
                GoodsAttribute = string.IsNullOrEmpty(x.GoodsAttribute) ? null : JsonConvert.DeserializeObject<List<GoogsAttribute>>(x.GoodsAttribute),
                GoodsName = x.GoodsName,
                GoodsWeight = x.GoodsWeight,
                GroupId = x.GroupId,
                Id = x.Id,
                OrderId = x.OrderId,
            }).ToList();


            result.GoodsInfo = result.GoodsInfo.GroupBy(x=>new{x.GroupId,x.GoodsAttribute,x.GoodsName,x.OrderId})
                .Select(x=>new WeighOrderGoodsDto()
                {
                    GoodsAttribute=x.FirstOrDefault()?.GoodsAttribute,
                    GoodsId=x.FirstOrDefault()?.GoodsId,
                    GoodsName=x.FirstOrDefault()?.GoodsName,
                    GoodsWeight= SqlFunc.AggregateSum(x.FirstOrDefault()?.GoodsWeight??0),
                    GroupId=x.FirstOrDefault()?.GroupId,
                    OrderId=x.FirstOrDefault()?.OrderId,
                })
                .ToList();

           //查询订单餐盘
           var diningPlate = await db.Queryable<BPA_WeighOrderDiningPlate>()
                .Where(x => x.WeighOrderId == inputDto.OrderId)
                .Select(x => new WeighOrderDiningPlateDto()
                {
                    DiningPlateId = x.DiningPlateId,
                    OrderId = inputDto.OrderId,
                }).ToListAsync();
            result.DiningPlateInfo = diningPlate;

            result.TotalAmount = result.GoodsInfo.Sum(x => x.TotalAmount);

            return result;
        }

        /// <summary>
        /// 修改订单状态
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateWeighOrderStates(WeighOrderUpdateDto inputDto)
        {
            //查询订单
            var result = await db.Queryable<BPA_WeighOrder>()
                 .Select(x => new WeighOrderDto
                 {
                     CreateAt = x.CreateAt,
                     CreateId = x.CreateId,
                     GroupId = x.GroupId,
                     Id = x.Id,
                     OrderNumber = x.OrderNumber,
                     PayStates = 0,
                     SceneId = x.SceneId,
                     States = 0,
                     Subject = x.Subject,
                     TotalAmount = x.TotalAmount,
                 })
                 .FirstAsync(x => x.Id == inputDto.OrderId);

            result.PayStates = inputDto.PayStates;
            result.States = inputDto.States;

            var res =await db.Updateable(result).ExecuteCommandAsync();

            return res > 0;
        }

        /// <summary>
        /// 添加 餐盘解绑记录
        /// </summary>
        /// <param name="inputDto">餐盘ID</param>
        /// <returns></returns>
        public async Task<bool> AddWeighOrderDiningPlateUnbindRecord(List<string> inputDto)
        {
            var list=new List<BPA_WeighOrderDiningPlateUnbindRecord>();

            foreach (var item in inputDto)
            {
                list.Add(new BPA_WeighOrderDiningPlateUnbindRecord()
                {
                    CreateAt=DateTime.Now,
                    DiningPlateId=item
                });
            }

            if (list.Count>0)
            {
                var res=await db.Updateable(list).ExecuteCommandAsync();
                return res > 0;
            }

            return false;
        }

        /// <summary>
        /// 获取餐盘绑定信息
        /// </summary>
        /// <param name="inputDto">餐盘ID</param>
        /// <returns></returns>
        public async Task<List<DiningPlateDto>> GetDiningPlateOccupyInfo(List<string> inputDto)
        {
            var data = await db.Queryable<BPA_WeighOrderDiningPlate, BPA_WeighOrder, BPA_WeighOrderDiningPlateUnbindRecord>
                    ((a, b, c) => new JoinQueryInfos(JoinType.Left, a.WeighOrderId == b.Id, JoinType.Left, a.DiningPlateId == c.DiningPlateId))
                .WhereIF(inputDto.Count > 0, (a, b, c) => inputDto.Contains(a.DiningPlateId))
                .Select((a, b, c) => new DiningPlateDto()
                {
                    DiningPlateId = a.DiningPlateId,
                    IsOccupy = b.States == 1 ? false : ((c == null || c.Id == null) ? false : true)
                })
                .ToListAsync();
            data = data.Where(x => x.IsOccupy == true).ToList();

            return data;

        }


        /// <summary>
        /// 更具openid获取称重订单
        /// </summary>
        /// <returns></returns>
        public async Task<PageUtil> GetWeighOrderByOpenId(GetWeighOrderByOpenIdInputDto inputDto)
        {
            int total = 0;
            var res = db.Queryable<BPA_WeighOrder>()
                .Where(x=>x.CreateId==inputDto.OpenId)
                .Select(t => new WeighOrderDto
                {
                    Id = t.Id.SelectAll(),
                }).ToPageList(inputDto.Current, inputDto.PageSize, ref total);

            foreach (var item in res)
            {
                //查询订单商品
                var orderGoods = await db.Queryable<BPA_WeighOrderGoods>()
                    .Where(x => x.OrderId == item.Id)
                    .ToListAsync();

                item.GoodsInfo = orderGoods.Select(x => new WeighOrderGoodsDto()
                {
                    GoodsId = x.GoodsId,
                    GoodsAttribute = string.IsNullOrEmpty(x.GoodsAttribute) ? null : JsonConvert.DeserializeObject<List<GoogsAttribute>>(x.GoodsAttribute),
                    GoodsName = x.GoodsName,
                    GoodsWeight = x.GoodsWeight,
                    GroupId = x.GroupId,
                    Id = x.Id,
                    OrderId = x.OrderId,
                }).ToList();


                item.GoodsInfo = item.GoodsInfo.GroupBy(x => new { x.GroupId, x.GoodsAttribute, x.GoodsName, x.OrderId })
                    .Select(x => new WeighOrderGoodsDto()
                    {
                        GoodsAttribute = x.FirstOrDefault()?.GoodsAttribute,
                        GoodsId = x.FirstOrDefault()?.GoodsId,
                        GoodsName = x.FirstOrDefault()?.GoodsName,
                        GoodsWeight = SqlFunc.AggregateSum(x.FirstOrDefault()?.GoodsWeight ?? 0),
                        GroupId = x.FirstOrDefault()?.GroupId,
                        OrderId = x.FirstOrDefault()?.OrderId,
                    })
                    .ToList();

                //查询订单餐盘
                var diningPlate = await db.Queryable<BPA_WeighOrderDiningPlate>()
                    .Where(x => x.WeighOrderId == item.Id)
                    .Select(x => new WeighOrderDiningPlateDto()
                    {
                        DiningPlateId = x.DiningPlateId,
                        OrderId = x.WeighOrderId,
                    }).ToListAsync();
                item.DiningPlateInfo = diningPlate;

                item.TotalAmount = item.GoodsInfo.Sum(x => x.TotalAmount);
            }


            return  new PageUtil()
            {
                Total = total,
                Data = res

            };
        }

    }
}
